/*
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001-2003 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Cactus" and "Apache Software
 *    Foundation" must not be used to endorse or promote products
 *    derived from this software without prior written permission. For
 *    written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */
package org.apache.cactus.sample.unit;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.StringWriter;

import javax.servlet.http.HttpServletResponse;

import org.apache.cactus.ServletTestCase;
import org.apache.cactus.WebRequest;
import org.apache.cactus.WebResponse;

/**
 * Tests that writes to the HTTP response.
 *
 * @author <a href="mailto:vmassol@apache.org">Vincent Massol</a>
 *
 * @version $Id: TestHttpResponse.java,v 1.6 2003/06/09 20:28:47 cmlenz Exp $
 */
public class TestHttpResponse extends ServletTestCase
{
    /**
     * Verify that it is possible to write to the servlet output stream.
     * 
     * @exception IOException on test failure
     */
    public void testWriteOutputStream() throws IOException
    {
        PrintWriter pw = response.getWriter();

        pw.println("should not result in an error");
    }

    /**
     * Verify that it is possible to write to the servlet output stream.
     *
     * @param theResponse the response from the server side.
     * 
     * @exception IOException on test failure
     */
    public void endWriteOutputStream(WebResponse theResponse) throws IOException
    {
        BufferedReader br = 
            new BufferedReader(new InputStreamReader(new DataInputStream(
            theResponse.getConnection().getInputStream())));

        assertEquals("should not result in an error", br.readLine());
    }

    //-------------------------------------------------------------------------

    /**
     * Verify that the <code>AsertUtils.getResponseAsString()</code> method
     * works with output text sent on multiple lines.
     * 
     * @exception IOException on test failure
     */
    public void testGetResponseAsStringMultiLines() throws IOException
    {
        PrintWriter pw = response.getWriter();

        response.setContentType("text/html");
        pw.println("<html><head/>");
        pw.println("<body>A GET request</body>");
        pw.println("</html>");
    }

    /**
     * Verify that the <code>AsertUtils.getResponseAsString()</code> method
     * works with output text sent on multiple lines.
     *
     * @param theResponse the response from the server side.
     * 
     * @exception IOException on test failure
     */
    public void endGetResponseAsStringMultiLines(WebResponse theResponse)
        throws IOException
    {
        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw);

        pw.println("<html><head/>");
        pw.println("<body>A GET request</body>");
        pw.println("</html>");

        String result = theResponse.getText();

        assertEquals(sw.toString(), result);

        pw.close();
    }

    //-------------------------------------------------------------------------

    /**
     * Verify that the <code>getTestAsArray()</code> method
     * works with output text sent on multiple lines. We also verify that
     * we can call it several times with the same result.
     * 
     * @exception IOException on test failure
     */
    public void testGetResponseAsStringArrayMultiLines() throws IOException
    {
        PrintWriter pw = response.getWriter();

        response.setContentType("text/html");
        pw.println("<html><head/>");
        pw.println("<body>A GET request</body>");
        pw.println("</html>");
    }

    /**
     * Verify that the <code>getTestAsArray()</code> method
     * works with output text sent on multiple lines. We also verify that
     * we can call it several times with the same result.
     *
     * @param theResponse the response from the server side.
     * 
     * @exception IOException on test failure
     */
    public void endGetResponseAsStringArrayMultiLines(WebResponse theResponse)
        throws IOException
    {
        String[] results1 = theResponse.getTextAsArray();
        String[] results2 = theResponse.getTextAsArray();

        assertTrue("Should have returned 3 lines of text but returned ["
            + results1.length + "]", results1.length == 3);
        assertEquals("<html><head/>", results1[0]);
        assertEquals("<body>A GET request</body>", results1[1]);
        assertEquals("</html>", results1[2]);

        assertTrue("Should have returned 3 lines of text but returned ["
            + results2.length + "]", results2.length == 3);
        assertEquals("<html><head/>", results2[0]);
        assertEquals("<body>A GET request</body>", results2[1]);
        assertEquals("</html>", results2[2]);
    }

    //-------------------------------------------------------------------------

    /**
     * Verify that a test case can get the request body by calling
     * <code>HttpServletRequest.getReader()</code>. In other words, verify that
     * internal parameters that Cactus passes from its client side to the
     * server do not affect this ability.
     *
     * @param theRequest the request object that serves to initialize the
     *                   HTTP connection to the server redirector.
     */
    public void beginGetReader(WebRequest theRequest)
    {
        theRequest.addParameter("test", "something", WebRequest.POST_METHOD);
    }

    /**
     * Verify that a test case can get the request body by calling
     * <code>HttpServletRequest.getReader()</code>. In other words, verify that
     * internal parameters that Cactus passes from its client side to the
     * server do not affect this ability.
     * 
     * @exception Exception on test failure
     */
    public void testGetReader() throws Exception
    {
        String buffer;
        StringBuffer body = new StringBuffer();

        BufferedReader reader = request.getReader();

        while ((buffer = reader.readLine()) != null)
        {
            body.append(buffer);
        }

        assertEquals("test=something", body.toString());
    }

    //-------------------------------------------------------------------------

    /**
     * Verify we can set and retrieve the content type.
     * 
     * @throws Exception If an unexpected error occurs
     */
    public void testSetContentType() throws Exception
    {
        response.setContentType("text/xml");

        // Although we don't assert the written content, this is needed to make
        // the test succeed on some versions of Orion. If the content is left 
        // empty, Orion will somehow reset the content-type to text/html. Sigh.
        PrintWriter pw = response.getWriter();
        pw.println("<?xml version=\"1.0\"?>");
        pw.println("<test></test>");
    }

    /**
     * Verify we can set and retrieve the content type.
     * 
     * @param theResponse the response from the server side.
     */
    public void endSetContentType(WebResponse theResponse)
    {
        assertEquals("text/xml", 
            theResponse.getConnection().getContentType());
    }
    
    //-------------------------------------------------------------------------

    /**
     * Verify that we can assert HTTP status code when it is a redirect and
     * that the client side of Cactus does not follow the redirect.
     * 
     * @exception IOException on test failure
     */
    public void testRedirect() throws IOException
    {
        response.sendRedirect("http://jakarta.apache.org");
    }

    /**
     * Verify that we can assert HTTP status code when it is a redirect and
     * that the client side of Cactus does not follow the redirect.
     *
     * @param theResponse the response from the server side.
     * 
     * @exception IOException on test failure
     */
    public void endRedirect(WebResponse theResponse) throws IOException
    {
        assertEquals(HttpServletResponse.SC_MOVED_TEMPORARILY, 
            theResponse.getStatusCode());
    }

    //-------------------------------------------------------------------------

    /**
     * Verify that we can assert HTTP status code when it is an error
     * status code (> 400).
     *
     * Note: HttpURLConnection will return a FileNotFoundException if the
     * status code is > 400 and the request does not end with a "/" !
     */
    public void testStatusCode()
    {
        response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
    }

    /**
     * Verify that we can assert HTTP status code when it is an error
     * status code (> 400).
     *
     * Note: HttpURLConnection will return a FileNotFoundException if the
     * status code is > 400 and the request does not end with a "/" !
     *
     * @param theResponse the response from the server side.
     * 
     * @exception IOException on test failure
     */
    public void endStatusCode(WebResponse theResponse) throws IOException
    {
        assertEquals(HttpServletResponse.SC_INTERNAL_SERVER_ERROR, 
            theResponse.getStatusCode());
    }
}
